import { expect } from 'chai';
import path from 'path';
import { run } from './jq';
import { FILTER_UNDEFINED_ERROR } from './command';
import { INVALID_PATH_ERROR, INVALID_JSON_PATH_ERROR } from './utils';
var PATH_ROOT = path.join(__dirname, '..');
var PATH_BIN = path.join(PATH_ROOT, './bin');
var PATH_ASTERISK_FIXTURE = path.join(PATH_ROOT, 'src', '*.js');
var PATH_FIXTURES = path.join(__dirname, '__test__', 'fixtures');
var PATH_JSON_FIXTURE = path.join(PATH_FIXTURES, '1.json');
var PATH_JS_FIXTURE = path.join(PATH_FIXTURES, '1.js');
var PATH_LARGE_JSON_FIXTURE = path.join(PATH_FIXTURES, 'large.json');
var PATH_VARIABLE_JSON_FIXTURE = path.join(PATH_FIXTURES, 'var.json');
var FIXTURE_JSON = require('./__test__/fixtures/1.json');
var FIXTURE_JSON_STRING = JSON.stringify(FIXTURE_JSON, null, 2);
var FILTER_VALID = '.repository.type';
var FILTER_INVALID = 'invalid';
var FILTER_WITH_VARIABLE = '[ . as $x | .user[] | {"user": ., "site": $x.site} ]';
var ERROR_INVALID_FILTER = /invalid/;
var CWD_OPTION = path.join(PATH_BIN, '../');
describe('jq core', function() {
    it('should fulfill its promise', function(done) {
        run(FILTER_VALID, PATH_JSON_FIXTURE).then(function(output) {
            expect(output).to.equal('"git"');
            done();
        }).catch(function(error) {
            done(error);
        });
    });
    it('should pass on an empty filter', function(done) {
        run('', PATH_JSON_FIXTURE).then(function(output) {
            var normalizedOutput = output.replace(/\r\n/g, '\n');
            expect(normalizedOutput).to.equal(FIXTURE_JSON_STRING);
            done();
        }).catch(function(error) {
            done(error);
        });
    });
    it('should pass on a null filter', function(done) {
        run(null, PATH_JSON_FIXTURE).then(function(output) {
            expect(output).to.equal('null');
            done();
        }).catch(function(error) {
            done(error);
        });
    });
    it('should fail on an invalid filter', function(done) {
        run(FILTER_INVALID, PATH_JSON_FIXTURE).catch(function(error) {
            expect(error).to.be.an.instanceof(Error);
            expect(error.message).to.match(ERROR_INVALID_FILTER);
            done();
        }).catch(function(error) {
            done(error);
        });
    });
    it('should fail on an undefined filter', function(done) {
        run(undefined, PATH_JSON_FIXTURE).catch(function(error) {
            expect(error).to.be.an.instanceof(Error);
            expect(error.message).to.equal(FILTER_UNDEFINED_ERROR);
            done();
        }).catch(function(error) {
            done(error);
        });
    });
    it('should fail on an invalid path', function(done) {
        run(FILTER_VALID, PATH_ASTERISK_FIXTURE).catch(function(error) {
            expect(error).to.be.an.instanceof(Error);
            expect(error.message).to.equal("".concat(INVALID_PATH_ERROR, ": \"").concat(PATH_ASTERISK_FIXTURE, "\""));
            done();
        }).catch(function(error) {
            done(error);
        });
    });
    it('should fail on an invalid json', function(done) {
        run(FILTER_VALID, PATH_JS_FIXTURE).catch(function(error) {
            expect(error).to.be.an.instanceof(Error);
            expect(error.message).to.equal("".concat(INVALID_JSON_PATH_ERROR, ": \"").concat(PATH_JS_FIXTURE, "\""));
            done();
        }).catch(function(error) {
            done(error);
        });
    });
    it('should not evaluate variables in the shell', function(done) {
        run(FILTER_WITH_VARIABLE, PATH_VARIABLE_JSON_FIXTURE, {
            output: 'json'
        }).then(function(obj) {
            expect(obj[0]).to.have.property('user');
            done();
        }).catch(function(error) {
            done(error);
        });
    });
    it('should allow inputs over max argument size limit', function(done) {
        run('.', PATH_LARGE_JSON_FIXTURE).then(function(output) {
            expect(output).to.be.a('string');
            done();
        }).catch(function(error) {
            done(error);
        });
    });
    it('should allow custom jqPath from function argument', function(done) {
        run(FILTER_VALID, PATH_JSON_FIXTURE, undefined, PATH_BIN).then(function(output) {
            expect(output).to.equal('"git"');
            done();
        }).catch(function(error) {
            done(error);
        });
    });
    it('should allow custom jqPath from env', function(done) {
        process.env.JQ_PATH = PATH_BIN;
        run(FILTER_VALID, PATH_JSON_FIXTURE).then(function(output) {
            expect(output).to.equal('"git"');
            done();
        }).catch(function(error) {
            done(error);
        });
        process.env.JQ_PATH = undefined;
    });
    it('should allow custom cwd', function(done) {
        run(FILTER_VALID, PATH_JSON_FIXTURE, undefined, undefined, CWD_OPTION).then(function(output) {
            expect(output).to.equal('"git"');
            done();
        }).catch(function(error) {
            done(error);
        });
    });
});
