import { expect } from 'chai';
import path from 'path';
import { run } from './jq';
import { optionDefaults, parseOptions } from './options';
import { INPUT_JSON_UNDEFINED_ERROR, INPUT_STRING_ERROR } from './command';
import { INVALID_PATH_ERROR, INVALID_JSON_PATH_ERROR } from './utils';
var PATH_FIXTURES = path.join(__dirname, '__test__', 'fixtures');
var PATH_JSON_FIXTURE = path.join(PATH_FIXTURES, '1.json');
var PATH_JSON_ARRAY_FIXTURE = path.join(PATH_FIXTURES, '2.json');
var PATH_SLURP_FIXTURE_1 = path.join(PATH_FIXTURES, 'slurp1.json');
var PATH_SLURP_FIXTURE_2 = path.join(PATH_FIXTURES, 'slurp2.json');
var PATH_SORT_FIXTURE = path.join(PATH_FIXTURES, 'sort.json');
var FIXTURE_JSON = require('./__test__/fixtures/1.json');
var FIXTURE_JSON_STRING = JSON.stringify(FIXTURE_JSON);
var FIXTURE_JSON_PRETTY = JSON.stringify(FIXTURE_JSON, null, 2);
var FIXTURE_JSON_ARRAY = require('./__test__/fixtures/2.json');
var FIXTURE_JSON_ARRAY_STRING = JSON.stringify(FIXTURE_JSON_ARRAY);
var FIXTURE_JSON_ARRAY_PRETTY = JSON.stringify(FIXTURE_JSON_ARRAY, null, 2);
var FILTER_VALID = '.repository.type';
var OPTION_DEFAULTS = {
    input: 'file',
    output: 'pretty',
    slurp: false,
    sort: false,
    raw: false,
    locations: [],
    color: false
};
var multiEOL = function(text) {
    return [
        text,
        text.replace(/\n/g, '\r\n')
    ];
};
describe('parse options', function() {
    it('should work with undefined options', function(done) {
        try {
            var result = parseOptions(undefined, FILTER_VALID, FIXTURE_JSON);
            expect(result).to.deep.equal([
                FILTER_VALID
            ]);
            done();
        } catch (e) {
            done(e);
        }
    });
});
describe('options', function() {
    it('defaults should be as expected', function() {
        expect(optionDefaults).to.deep.equal(OPTION_DEFAULTS);
    });
    describe('input', function() {
        describe('file', function() {
            it('should accept a file path as input', function(done) {
                run('.', PATH_JSON_FIXTURE, {
                    input: 'file'
                }).then(function(output) {
                    expect(output).to.not.equal(null);
                    done();
                }).catch(function(error) {
                    done(error);
                });
            });
            it('should accept an array with multiple file paths as input', function(done) {
                run('.', [
                    PATH_JSON_FIXTURE,
                    PATH_JSON_FIXTURE
                ], {
                    input: 'file'
                }).then(function(output) {
                    expect(output).to.not.equal(null);
                    done();
                }).catch(function(error) {
                    done(error);
                });
            });
            it('should fail on an empty path', function(done) {
                run('.', '', {
                    input: 'file'
                }).catch(function(error) {
                    expect(error).to.be.an.instanceof(Error);
                    expect(error.message).to.equal("".concat(INVALID_JSON_PATH_ERROR, ": \"\""));
                    done();
                }).catch(function(error) {
                    done(error);
                });
            });
            it('should fail on a null path', function(done) {
                run('.', null, {
                    input: 'file'
                }).catch(function(error) {
                    expect(error).to.be.an.instanceof(Error);
                    expect(error.message).to.equal("".concat(INVALID_PATH_ERROR, ": \"null\""));
                    done();
                }).catch(function(error) {
                    done(error);
                });
            });
            it('should fail on an undefined path', function(done) {
                run('.', undefined, {
                    input: 'file'
                }).catch(function(error) {
                    expect(error).to.be.an.instanceof(Error);
                    expect(error.message).to.equal("".concat(INVALID_PATH_ERROR, ": \"undefined\""));
                    done();
                }).catch(function(error) {
                    done(error);
                });
            });
        });
        describe('json', function() {
            it('should accept a json object as input', function(done) {
                run('.', FIXTURE_JSON, {
                    input: 'json'
                }).then(function(output) {
                    expect(output).to.not.equal(null);
                    done();
                }).catch(function(error) {
                    done(error);
                });
            });
            it('should accept an array object', function(done) {
                run('.', FIXTURE_JSON_ARRAY, {
                    input: 'json'
                }).then(function(output) {
                    expect(output).to.not.equal(null);
                    done();
                }).catch(function(error) {
                    done(error);
                });
            });
            it('should accept an empty string as json object', function(done) {
                run('.', '', {
                    input: 'json'
                }).then(function(output) {
                    expect(output).to.equal('""');
                    done();
                }).catch(function(error) {
                    done(error);
                });
            });
            it('should accept a null json object', function(done) {
                run('.', null, {
                    input: 'json'
                }).then(function(output) {
                    expect(output).to.equal('null');
                    done();
                }).catch(function(error) {
                    done(error);
                });
            });
            it('should fail on an undefined json object', function(done) {
                run('.', undefined, {
                    input: 'json'
                }).catch(function(error) {
                    expect(error).to.be.an.instanceof(Error);
                    expect(error.message).to.equal(INPUT_JSON_UNDEFINED_ERROR);
                    done();
                }).catch(function(error) {
                    done(error);
                });
            });
        });
        describe('string', function() {
            it('should accept a json string as input', function(done) {
                run('.', FIXTURE_JSON_STRING, {
                    input: 'string'
                }).then(function(output) {
                    expect(output).to.not.equal(null);
                    done();
                }).catch(function(error) {
                    done(error);
                });
            });
            it('should accept a json array as input', function(done) {
                run('.', FIXTURE_JSON_ARRAY_STRING, {
                    input: 'string'
                }).then(function(output) {
                    expect(output).to.not.equal(null);
                    done();
                }).catch(function(error) {
                    done(error);
                });
            });
            it('should fail on an empty string', function(done) {
                run('.', '', {
                    input: 'string'
                }).catch(function(error) {
                    expect(error).to.be.an.instanceof(Error);
                    expect(error.message).to.equal("".concat(INPUT_STRING_ERROR, ": \"\""));
                    done();
                }).catch(function(error) {
                    done(error);
                });
            });
            it('should fail on a null string', function(done) {
                run('.', null, {
                    input: 'string'
                }).catch(function(error) {
                    expect(error).to.be.an.instanceof(Error);
                    expect(error.message).to.equal("".concat(INPUT_STRING_ERROR, ": \"null\""));
                    done();
                }).catch(function(error) {
                    done(error);
                });
            });
            it('should fail on an undefined string', function(done) {
                run('.', undefined, {
                    input: 'string'
                }).catch(function(error) {
                    expect(error).to.be.an.instanceof(Error);
                    expect(error.message).to.equal("".concat(INPUT_STRING_ERROR, ": \"undefined\""));
                    done();
                }).catch(function(error) {
                    done(error);
                });
            });
        });
    });
    describe('output', function() {
        describe('json', function() {
            it('should return a stringified json', function(done) {
                run('.', PATH_JSON_FIXTURE, {
                    output: 'json'
                }).then(function(obj) {
                    expect(obj).to.deep.equal(FIXTURE_JSON);
                    done();
                }).catch(function(error) {
                    done(error);
                });
            });
            it('it should return a string if the filter calls an array', function(done) {
                run('.contributors[]', PATH_JSON_FIXTURE, {
                    output: 'json'
                }).then(function(obj) {
                    expect(obj).to.be.oneOf(multiEOL(JSON.stringify(FIXTURE_JSON.contributors[0], null, 2) + '\n' + JSON.stringify(FIXTURE_JSON.contributors[1], null, 2)));
                    done();
                }).catch(function(error) {
                    done(error);
                });
            });
        });
        describe('string', function() {
            it('should return a minified json string', function(done) {
                run('.', PATH_JSON_FIXTURE, {
                    output: 'string'
                }).then(function(output) {
                    expect(output).to.equal(FIXTURE_JSON_STRING);
                    done();
                }).catch(function(error) {
                    done(error);
                });
            });
            it('should return a minified json array string', function(done) {
                run('.', PATH_JSON_ARRAY_FIXTURE, {
                    output: 'string'
                }).then(function(output) {
                    expect(output).to.equal(FIXTURE_JSON_ARRAY_STRING);
                    done();
                }).catch(function(error) {
                    done(error);
                });
            });
        });
        describe('compact', function() {
            it('should return a minified json string', function(done) {
                run('.', PATH_JSON_FIXTURE, {
                    output: 'compact'
                }).then(function(output) {
                    expect(output).to.equal(FIXTURE_JSON_STRING);
                    done();
                }).catch(function(error) {
                    done(error);
                });
            });
        });
        describe('pretty', function() {
            it('should return a prettified json string', function(done) {
                run('.', PATH_JSON_FIXTURE, {
                    output: 'pretty'
                }).then(function(output) {
                    expect(output).to.be.oneOf(multiEOL(FIXTURE_JSON_PRETTY));
                    done();
                }).catch(function(error) {
                    done(error);
                });
            });
            it('should return a prettified json array string', function(done) {
                run('.', PATH_JSON_ARRAY_FIXTURE, {
                    output: 'pretty'
                }).then(function(output) {
                    expect(output).to.be.oneOf(multiEOL(FIXTURE_JSON_ARRAY_PRETTY));
                    done();
                }).catch(function(error) {
                    done(error);
                });
            });
        });
    });
    describe('slurp: true', function() {
        it('should run the filter once on multiple objects as input', function(done) {
            run('.', [
                PATH_SLURP_FIXTURE_1,
                PATH_SLURP_FIXTURE_2
            ], {
                output: 'json',
                slurp: true
            }).then(function(output) {
                expect(output).to.be.an('array');
                expect(output).to.have.lengthOf(2);
                done();
            }).catch(function(error) {
                done(error);
            });
        });
    });
    describe('sort: false', function() {
        it('keys should be ordered the same as in input file', function(done) {
            run('.', [
                PATH_SORT_FIXTURE
            ], {
                output: 'string',
                sort: false
            }).then(function(output) {
                expect(output).to.be.a('string');
                expect(output).to.eql('{"z":1,"a":2}');
                done();
            }).catch(function(error) {
                done(error);
            });
        });
    });
    describe('sort: true', function() {
        it('keys should be ordered alphabetically', function(done) {
            run('.', [
                PATH_SORT_FIXTURE
            ], {
                output: 'string',
                sort: true
            }).then(function(output) {
                expect(output).to.be.a('string');
                expect(output).to.eql('{"a":2,"z":1}');
                done();
            }).catch(function(error) {
                done(error);
            });
        });
    });
    describe('raw: true', function() {
        it('output should be able to output raw strings instead of JSON texts', function(done) {
            run('.name', PATH_JSON_FIXTURE, {
                input: 'file',
                raw: true
            }).then(function(output) {
                expect(output).to.equal('node-jq');
                done();
            }).catch(function(e) {
                return done(e);
            });
        });
        it('output should be able to output JSON texts instead of raw strings', function(done) {
            run('.name', PATH_JSON_FIXTURE, {
                input: 'file',
                raw: false
            }).then(function(output) {
                expect(output).to.equal('"node-jq"');
                done();
            }).catch(function(e) {
                return done(e);
            });
        });
    });
});
